;;; navi2ch-message.el --- write message module for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'navi2ch-article)
(require 'navi2ch-board)

(defvar navi2ch-message-buffe-name "*navi2ch message*")
(defvar navi2ch-message-backup-buffer-name "*navi2ch message backup*")
(defvar navi2ch-message-current-article-buffer nil)
(defvar navi2ch-message-current-article nil)
(defvar navi2ch-message-current-board nil)
(defvar navi2ch-message-new-message-p nil)
(defvar navi2ch-message-window-configuration nil)
(defvar navi2ch-message-user-name
  (if (featurep 'xemacs)
      "$BL>L5$7$5$s!w#X#E#m#a#c#s(B"
    "$BL>L5$7$5$s!w#E#m#a#c#s(B")
  "$BL>A0(B")
(defvar navi2ch-message-ask-before-send t)
(defvar navi2ch-message-ask-before-kill t)
(defvar navi2ch-message-wait-time 1
  "$BAw$C$?8e(B sync $B$9$kA0$KBT$D;~4V(B($BIC(B)")

(defvar navi2ch-message-remember-user-name t)
(defvar navi2ch-message-cite-prefix "> " "$B0zMQ$9$k$H$-$N@\F,<-(B")
(defvar navi2ch-message-mode-map nil)

(unless navi2ch-message-mode-map
  (setq navi2ch-message-mode-map (make-sparse-keymap))
  (define-key navi2ch-message-mode-map "\C-c\C-c" 'navi2ch-message-send-message)
  (define-key navi2ch-message-mode-map "\C-c\C-k" 'navi2ch-message-exit)
  (define-key navi2ch-message-mode-map "\C-c\C-y" 'navi2ch-message-cite-original)
  (define-key navi2ch-message-mode-map "\C-c\y" 'navi2ch-message-cite-original-from-number))

(defun navi2ch-message-write-message (board article &optional new)
  (when (or (not (get-buffer navi2ch-message-buffe-name))
			(navi2ch-message-kill-message))
	(setq navi2ch-message-window-configuration
		  (current-window-configuration))
	(delete-other-windows)
	(split-window-vertically)
	(other-window 1)
	(setq navi2ch-message-current-article article)
	(setq navi2ch-message-current-board board)
	(setq navi2ch-message-new-message-p new)
	(setq navi2ch-message-current-article-buffer
		  (if new nil (current-buffer)))
	(switch-to-buffer (get-buffer-create navi2ch-message-buffe-name))
	(navi2ch-message-mode)
	(erase-buffer)
	(navi2ch-message-insert-header new)))

(defun navi2ch-message-insert-header (new)
  (when new
    (insert "Subject: \n"))
  (insert "From: "
	  (or (cdr (assq 'name navi2ch-message-current-article))
              navi2ch-message-user-name) "\n"
          "Mail: \n"
          (make-string (1- (window-width)) ?-) "\n"))
  
(defun navi2ch-message-send-message ()
  (interactive)
  (when (or (not navi2ch-message-ask-before-send)
            (y-or-n-p "send message?"))
    (save-excursion
      (let (subject from mail message)
        (goto-char (point-min))
        (when (re-search-forward "Subject: \\([^\n]*\\)\n" nil t)
          (setq subject (match-string 1)))
        (re-search-forward "From: \\([^\n]*\\)\n")
        (setq from (match-string 1))
        (when navi2ch-message-remember-user-name
          (setq navi2ch-message-user-name from))
        (when (not navi2ch-message-new-message-p)
          (navi2ch-message-set-name from))
        (re-search-forward "Mail: \\([^\n]*\\)\n")
        (setq mail (match-string 1))
        (forward-line 1)
        (setq message (buffer-substring (point) (point-max)))
        (let ((str (buffer-substring-no-properties
                    (point-min) (point-max))))
          (save-excursion
            (set-buffer (get-buffer-create
                         navi2ch-message-backup-buffer-name))
            (erase-buffer)
            (insert str)
            (bury-buffer)))
        (when (navi2ch-net-send-message
               from mail message subject
               navi2ch-message-current-board
               navi2ch-message-current-article)
          (sleep-for navi2ch-message-wait-time)
          (save-excursion
            (if navi2ch-message-new-message-p
                (progn
                    (set-buffer navi2ch-board-buffer-name)
                    (navi2ch-board-sync))
              (set-buffer (navi2ch-article-current-buffer))
              (navi2ch-article-sync))))))
    (navi2ch-message-exit 'after-send)))

(defun navi2ch-message-set-name (name)
  (save-excursion
    (set-buffer navi2ch-message-current-article-buffer)
    (setq navi2ch-article-current-article
          (navi2ch-put-alist 'name name
                             navi2ch-article-current-article))))

(defun navi2ch-message-cite-original ()
  "$B0zMQ$9$k(B"
  (interactive)
  (navi2ch-message-cite-original-from-number
   (save-excursion
     (set-buffer (navi2ch-article-current-buffer))
     (navi2ch-article-get-current-number))))

(defun navi2ch-message-cite-original-from-number (num)
  "$BHV9f$rA*$s$G!"0zMQ$9$k!#(B"
  (interactive "ninput number: ")
  (insert ">>" (number-to-string num) "\n")
  (let ((point (point)))
    (insert
     (save-excursion
       (set-buffer (navi2ch-article-current-buffer))
       (cdr (assq 'data (navi2ch-article-get-message num))))
     "\n")
    (string-rectangle point (point) navi2ch-message-cite-prefix)))
    
(defun navi2ch-message-exit (&optional after-send)
  (interactive)
  (when (navi2ch-message-kill-message after-send)
    ;; $B$`$%!"(Bset-window-configuration $B$r;H$&$H%+!<%=%k0LCV$,JQ$K$J$k$s$+$$!)(B
	(set-window-configuration navi2ch-message-window-configuration)
    (when (and (not navi2ch-message-new-message-p)
               after-send)
      (set-buffer (navi2ch-article-current-buffer))
      (navi2ch-article-load-number))))

(defun navi2ch-message-kill-message (&optional no-ask)
  (when (or no-ask
			(not navi2ch-message-ask-before-kill)
			(y-or-n-p "kill current message?"))
	(kill-buffer navi2ch-message-buffe-name)
	t))

(defun navi2ch-message-mode ()
  (interactive)
  (setq major-mode 'navi2ch-message-mode)
  (setq mode-name "Navi2ch Message")
  (use-local-map navi2ch-message-mode-map)
  (run-hooks 'navi2ch-message-mode-hook)
  (force-mode-line-update))

(provide 'navi2ch-message)

;;; navi2ch-message.el ends here