;;; navi2ch-search.el --- Search Module for Navi2ch

;; Copyright (C) 2001 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: 2ch, network

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(require 'cl)
(require 'navi2ch-board-misc)
(require 'navi2ch-board)

(defvar navi2ch-search-mode-map nil)
(unless navi2ch-search-mode-map
  (setq navi2ch-search-mode-map
        (copy-keymap navi2ch-bm-mode-map)))
  
(defvar navi2ch-search-searched-subjedt-list nil)
(defvar navi2ch-search-board
  '((name . "$B8!:w0lMw(B")
    (type . search)
    (id . "search")))

;;; navi2ch-bm callbacks
(defun navi2ch-search-set-property (begin end item)
  (put-text-property begin end 'item item))

(defun navi2ch-search-get-property (point)
  (get-text-property point 'item))

(defun navi2ch-search-get-board (item)
  (car item))

(defun navi2ch-search-get-article (item)
  (cdr item))

(defun navi2ch-search-exit ())

;;; navi2ch-search functions
(defun navi2ch-search-insert-subjects ()
  (let ((i 1))
    (dolist (x navi2ch-search-searched-subjedt-list)
      (let ((article (navi2ch-search-get-article x))
            (board (navi2ch-search-get-board x)))
        (navi2ch-bm-insert-subject
         x i
         (cdr (assq 'subject article))
         (format "[%s]" (cdr (assq 'name board))))
        (setq i (1+ i))))))

(defun navi2ch-search-board-subject-regexp (board-list regexp)
  (let (list)
    (dolist (board board-list)
      (let ((file (navi2ch-board-get-file-name board)))
        (when (file-exists-p file)
          (let (rep)
            (with-temp-buffer
              (navi2ch-insert-file-contents file)
              (goto-char (point-min))
              (setq rep (navi2ch-board-regexp-test))
              (while (re-search-forward regexp nil t)
                (beginning-of-line)
                (looking-at rep)
                (setq list (cons (cons board
                                       (navi2ch-board-get-matched-article))
                                 list))
                (forward-line)))))))
    (nreverse list)))

(defun navi2ch-search-mode ()
  "\\{navi2ch-search-mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'navi2ch-search-mode)
  (setq mode-name "Navi2ch Search")
  (setq buffer-read-only t)
  (use-local-map navi2ch-search-mode-map)
  (run-hooks 'navi2ch-search-mode-hook))

(defun navi2ch-search ()
  (navi2ch-search-mode)
  (navi2ch-bm-setup 'navi2ch-search)
  (let ((buffer-read-only nil))
    (erase-buffer)
    (save-excursion
      (navi2ch-search-insert-subjects))))
  
(defun navi2ch-search-all ()
  (interactive)
  (let ((alist navi2ch-list-category-list))
    (setq alist
          (if (assoc navi2ch-list-navi2ch-category-name alist)
              (delq (assoc navi2ch-list-navi2ch-category-name alist) 
                    (copy-list alist))
            alist))
    (navi2ch-search-subject-subr
     (navi2ch-list-get-board-name-list navi2ch-list-category-list))))

(defun navi2ch-search-subject-subr (board-list)
  (setq navi2ch-search-searched-subjedt-list
        (navi2ch-search-board-subject-regexp
         board-list (read-string "Subject regexp: ")))
  (navi2ch-bm-select-board navi2ch-search-board))

(provide 'navi2ch-search)
;;; navi2ch-search.el ends here