;;; navi2ch-util.el --- useful utilities for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'cl)
(require 'regexp-opt)

(defvar navi2ch-directory "~/.navi2ch")
(defvar navi2ch-uudecode-program "uudecode")
(defvar navi2ch-uudecode-args nil)

(defvar navi2ch-replace-html-tag-alist
  '(("&gt;" . ">")
    ("&lt;" . "<")
    ("&quot;" . "\"")
	("&nbsp;" . " ")
    ("$B!w!.(B" . ","))
  "$BCV49$9$k(B html $B$N%?%0$NO"A[%j%9%H(B($B@55,I=8=$O;H$($J$$(B)")

(defvar navi2ch-replace-html-tag-regexp
  (regexp-opt (mapcar
               'car
               navi2ch-replace-html-tag-alist))
  "$BCV49$9$k(B html $B$N%?%0$N@55,I=8=(B
`navi2ch-replace-html-tag-alist' $B$+$i@8@.$5$l$k(B")

(defsubst navi2ch-replace-string (rep new str &optional all)
  (if all
      (let (start)
        (while (setq start (string-match rep str start))
          (setq str (replace-match new nil nil str))))
    (when (string-match rep str)
      (setq str (replace-match new nil nil str))))
  str)
  
(defun navi2ch-bigint-int-to-list (i)
  (if (listp i)
      i
    (mapcar (lambda (x)
              (- x 48))
            (string-to-list (int-to-string i)))))

(defun navi2ch-bigint-multiply (a b)
  (setq a (reverse (navi2ch-bigint-int-to-list a))
        b (reverse (navi2ch-bigint-int-to-list b)))
  (let (list c)
    (dolist (y b)
      (let ((z 0))
        (setq list (cons
                    (append c (mapcar
                               (lambda (x)
                                 (let (w)
                                   (setq w (+ (* x y) z))
                                   (setq z (/ w 10))
                                   (mod w 10))) a)
                            (if (> z 0) (list z)))
                    list)))
      (setq c (cons 0 c)))
    (let (list2)
      (dolist (x list)
        (setq list2 (navi2ch-bigint-add list2 (reverse x))))
      list2)))

(defun navi2ch-bigint-add (a b)
  (setq a (reverse (navi2ch-bigint-int-to-list a))
        b (reverse (navi2ch-bigint-int-to-list b)))
  (let ((x 0) list)
    (while (or a b)
      (let (y)
        (setq y (+ (or (car a) 0) (or (car b) 0) x))
        (setq x (/ y 10))
        (setq list (cons (mod y 10) list))
        (setq a (cdr a)
              b (cdr b))))
    list))

(defun navi2ch-insert-file-contents (file &optional begin end)
  (let ((coding-system-for-read navi2ch-net-coding-system)
        (coding-system-for-write navi2ch-net-coding-system))
    (insert-file-contents file nil begin end)))

(defun navi2ch-expand-file-name (file)
  (expand-file-name file navi2ch-directory))
  
(defun navi2ch-uudecode-region (start end)
  (interactive "r")
  (let (dir)
    (save-window-excursion
      (delete-other-windows)
      (setq dir (read-file-name "directory name: ")))
    (unless (file-directory-p dir)
      (error "%s is not directory" dir))
    
    (let ((default-directory dir)
          (coding-system-for-read 'binary)
          (coding-system-for-write 'binary)
          rc)
      (setq rc (apply
                'call-process-region
                start end
                navi2ch-uudecode-program
                nil nil nil
                navi2ch-uudecode-args))
      (when (not (= rc 0))
        (error "uudecode error")))))

;; (defun navi2ch-read-number (prompt)
;;   "$B?t;z$r(B minibuffer $B$+$iFI$_9~$`(B"
;;   (catch 'loop
;;     (while t
;;       (let (elt)
;;         (setq elt (read-string prompt init history default))
;;         (cond ((string= elt "")
;;                (throw 'loop nil))
;;               ((string-match "^[ \t]*0+[ \t]*$" elt)
;;                (throw 'loop 0))
;;               ((not (eq (string-to-number elt) 0))
;;                (throw 'loop (string-to-int elt)))))
;;       (message "Please enter a number.")
;;       (sit-for 1))))

(defsubst navi2ch-replace-html-tag (str)
  (unless (string= str "")
    (setq str (navi2ch-replace-string " *<br> " "\n" str t))
    (setq str (navi2ch-replace-string "<[^<]+>" "" str t))
    (setq str (navi2ch-replace-string "&amp" "&" str t))
    (let (start)
      (while (setq start (string-match
                          navi2ch-replace-html-tag-regexp
                          str start))
        (let ((new (cdr (assoc (match-string 0 str)
                               navi2ch-replace-html-tag-alist))))
          (setq str (replace-match new nil nil str)))))
    (setq str (navi2ch-replace-string "&#[0-9]+;" "$B".(B" str t)))
  str)

(defsubst navi2ch-replace-html-tag-with-temp-buffer (str)
  (with-temp-buffer
    (insert str)
    (goto-char (point-min))
    (while (re-search-forward " *<br> " nil t)
      (replace-match "\n"))
    (goto-char (point-min))
    (while (re-search-forward "<[^<]+>" nil t)
      (replace-match ""))
    (goto-char (point-min))
    (while (re-search-forward "&amp" nil t)
      (replace-match "&"))
    (goto-char (point-min))
    (while (re-search-forward navi2ch-replace-html-tag-regexp nil t)
      (replace-match (cdr (assoc (match-string 0)
                                 navi2ch-replace-html-tag-alist))))
    (goto-char (point-min))
    (while (re-search-forward "&#[0-9]+;" nil t)
      (replace-match "$B".(B"))
    (buffer-string)))
      
      
(defun navi2ch-url-to-board (url)
  (let (uri id board)
    (cond ((string-match
            "http://\\([^/]+\\)/test/read\\.cgi.*bbs=\\([^&]+\\)" url)
           (setq id (match-string 2 url)
                 uri (format "http://%s/%s/" (match-string 1 url) id)))
          ((string-match "http://\\([^/]+\\)/\\([^/]+\\)" url)
           (setq id (match-string 2 url)
                 uri (format "http://%s/%s/" (match-string 1 url) id))))
    (when id
      (dolist (x (navi2ch-list-get-board-name-list
                  navi2ch-list-category-list))
        (when (string= (cdr (assq 'uri x)) uri)
          (setq board x)))
      (unless board
        (setq board (list (cons 'uri uri)
                          (cons 'id id))))
      board)))

(defun navi2ch-url-to-article (url)
  (when (string-match
         "http://[^/]+/test/read\\.cgi.*&key=\\([0-9]+\\)" url)
    (let ((list (list (cons 'artid (match-string 1 url)))))
      (when (string-match "&st=\\([0-9]+\\)" url)
        (setq list (cons (cons 'number
                               (string-to-number (match-string 1 url)))
                         list)))
      list)))

(defun navi2ch-article-to-url (board article &optional start end nofirst)
  "BOARD, ARTICLE $B$+$i(B url $B$KJQ49!#(B
START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
  (let ((url (cdr (assq 'uri board))))
    (setq url (concat
               (navi2ch-replace-string "\\(http://[^/]+/\\).+"
                                       "\\1" url)
               "test/read.cgi?bbs="
               (cdr (assq 'id board))
               "&key="
               (cdr (assq 'artid article))
               (when start
                 (format "&st=%d" start))
               (when end
                 (format "&to=%d" end))
               (when nofirst
                 "&nofirst=true")))
	url))

(defun navi2ch-board-to-url (board)
  "BOARD $B$+$i(B url $B$KJQ49(B"
  (concat (cdr (assq 'uri board)) "index2.html"))
							   
(defun navi2ch-2ch-url-p (url)
  (let (list)
    (setq list
          (mapcar
           (lambda (x)
             (let ((str (cdr (assq 'uri x))))
               (and str
                    (string-match "http://\\([^/]+\\)" str)
                    (match-string 1 str))))
           (navi2ch-list-get-board-name-list
            navi2ch-list-category-list)))
    (when (string-match "http://\\([^/]+\\)" url)
      (setq url (match-string 1 url))
      (member url list))))
                      
  
;; from apel
(defun navi2ch-put-alist (item value alist)
  "Modify ALIST to set VALUE to ITEM.
If there is a pair whose car is ITEM, replace its cdr by VALUE.
If there is not such pair, create new pair (ITEM . VALUE) and
return new alist whose car is the new pair and cdr is ALIST.
\[tomo's ELIS like function]"
  (let ((pair (assoc item alist)))
    (if pair
	(progn
	  (setcdr pair value)
	  alist)
      (cons (cons item value) alist))))

(defun navi2ch-next-property (point prop)
  (when (get-text-property point prop)
    (setq point (next-single-property-change point prop)))
  (when point
    (setq point (next-single-property-change point prop)))
  point)

(defun navi2ch-previous-property (point prop)
  (when (get-text-property point prop)
    (setq point (previous-single-property-change point prop)))
  (when point
    (unless (get-text-property (1- point) prop)
      (setq point (previous-single-property-change point prop)))
    (when point
      (1- point))))

(defun navi2ch-set-minor-mode (mode name map)
  (make-variable-buffer-local mode)
  (unless (assq mode minor-mode-alist)
    (setq minor-mode-alist
          (cons (list mode name) minor-mode-alist)))
  (unless (assq mode minor-mode-map-alist)
    (setq minor-mode-map-alist 
          (cons (cons mode map) minor-mode-map-alist))))

;; from apel
(if (featurep 'xemacs)
    (defun navi2ch-set-buffer-multibyte (flag)
      flag)
  (defun navi2ch-set-buffer-multibyte (flag)
    (set-buffer-multibyte flag)))

;; from apel
(if (featurep 'xemacs)
    (defun navi2ch-string-as-unibyte (string)
      string)
  (defun navi2ch-string-as-unibyte (string)
    (string-as-unibyte string)))

(defun navi2ch-get-major-mode (buffer)
  (when (get-buffer buffer)
    (save-excursion
      (set-buffer buffer)
      major-mode)))

(provide 'navi2ch-util)